/***************************************************************************
 *  Copyright (C) 1995-2011 OSS Nokalva, Inc.  All rights reserved.
 ***************************************************************************
 *  THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC.
 *  AND MAY BE USED ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC.
 *  THIS FILE MAY NOT BE DISTRIBUTED.
 ***************************************************************************
 */

/*

This sample program demonstrates decoding of PDU containing open type fields
without using automatic decoding

*/

#include <stdlib.h>
#include <stdio.h>
#include "ranap.h"

#define PDU_NUMBER RANAP_PDU_PDU
#define CTLTBL ranap

unsigned char encodedMsg[] = {
    0x00 ,0x00 ,0x40 ,0x35 ,0x00 ,0x00 ,0x02 ,0x00 ,
    0x36 ,0x40 ,0x1F ,0x00 ,0x00 ,0x01 ,0x00 ,0x35 ,
    0x00 ,0x13 ,0x00 ,0xFE ,0x00 ,0x48 ,0x00 ,0x00 ,
    0x01 ,0x1E ,0x00 ,0x00 ,0x08 ,0x91 ,0xA0 ,0x00 ,
    0x00 ,0x12 ,0x34 ,0x56 ,0x78 ,0x40 ,0x03 ,0x10 ,
    0x00 ,0x09 ,0x00 ,0x29 ,0x40 ,0x0B ,0x00 ,0x00 ,
    0x01 ,0x00 ,0x28 ,0x40 ,0x04 ,0x00 ,0x01 ,0x02 ,
    0x00 
};

void decode_AssignmentRequestIEs(OssGlobal *world, struct _seqof119 *input);
void decode_ReleaseItemIEs(OssGlobal *world, struct _seqof87 *input);
void decode_ContainerPair(OssGlobal *world, ProtocolIE_ContainerPair input);

int main(void)
{
    /* buffer containing encoded data */
    OssBuf	buf;

    /* pointer to decoded message */
    RANAP_PDU	*decodedMsg = NULL;

    /* identifies PDU to decode */
    int		pdunum = PDU_NUMBER;

    /* OSS global structure */
    OssGlobal	world;

    /* current opentype to decode */
    RANAP_ELEMENTARY_PROCEDURES_InitiatingMessage	*opentype;

    ossinit(&world, CTLTBL);    /* initialize OSS environment */

    ossSetEncodingRules(&world, OSS_PER_ALIGNED);
    ossSetFlags(&world, DEBUGPDU);
    buf.length = sizeof(encodedMsg);
    buf.value = encodedMsg;
    
    ossPrint(&world, "The decoder trace messages ...\n");
    
    ossDecode(&world, &pdunum, &buf, (void **)&decodedMsg);

    switch (decodedMsg->choice) {
	case initiatingMessage_chosen:
	    opentype = &decodedMsg->u.initiatingMessage.value;
	    buf.length = opentype->encoded.length;
	    buf.value = opentype->encoded.value;
	    pdunum = RAB_AssignmentRequest_PDU;
            ossPrint(&world, "\nDecoding the InitiatingMessage.value ...\n");
	    ossDecode(&world, &pdunum, &buf,
		    (void **)&opentype->decoded.pdu_Iu_ReleaseCommand);
	    if (pdunum == RAB_AssignmentRequest_PDU) {
		opentype->pduNum = RAB_AssignmentRequest_PDU;
                ossPrint(&world, "\nDecoding opentypes nested in the InitiatingMessage.value.protocolIEs ...\n");
		decode_AssignmentRequestIEs(&world, opentype->decoded.pdu_RAB_AssignmentRequest->protocolIEs);

		if (opentype->decoded.pdu_RAB_AssignmentRequest->bit_mask & RAB_AssignmentRequest_protocolExtensions_present) {
		    /* decode RAB_AssignmentRequest_protocolExtensions if present */
		}
	    } else  {
		/* more checks for other PDUs that can occur in the initiatingMessage */
	    }
            break;
	case successfulOutcome_chosen:
	    /* Do manual decoding of opentypes, nested in the
	       UnsuccessulOutcome
	    */
	    break;
	case unsuccessfulOutcome_chosen:
	    /* Do manual decoding of opentypes, nested in the
	       UnsuccessulOutcome
	    */
	    break;
	case outcome_chosen:
	    /* Do manual decoding of opentypes, nested in the
	       UnsuccessulOutcome
	    */   
	    break;
    }
    ossPrint(&world, "\nThe RANAP message decoded:\n");
    pdunum = PDU_NUMBER;
    ossPrintPDU(&world, pdunum, decodedMsg);
    ossFreePDU(&world, pdunum, decodedMsg);

    ossterm(&world);              /* cleanup OSS environment */

    return 0;
}

/*
 * Function:            decode_AssignmentRequestIEs() decodes protocolIEs PDU
 *
 * Parameters:
 *
 *   "world"        ... pointer to OssGlobal structure.
 *   "input"        ... pointer to the list of protocolIEs to decode
 *
 * Return value:        none.
 */
void decode_AssignmentRequestIEs(OssGlobal *world, struct _seqof119 *input)
{
    /* pointer to opentype to decode */
    RAB_AssignmentRequestIEs_Value *opentype = &input->value.value;

    /* decoder input buffer */
    OssBuf	buf;

    /* number of PDU to decode */
    int		pdunum;

    /* pointer to next item in RAB_ReleaseList */
    RAB_ReleaseList next;

    /* pointer to next pair in RAB_SetupOrModifyList */
    RAB_SetupOrModifyList next_pair;

    buf.length = opentype->encoded.length;
    buf.value = opentype->encoded.value;

    pdunum = input->next ? RAB_SetupOrModifyList_PDU : RAB_ReleaseList_PDU;
    ossDecode(world, &pdunum, &buf, (void **)&opentype->decoded.pdu_RAB_SetupOrModifyList);
    opentype->pduNum = pdunum;
    
    /* decode list of IEs recursively */
    if (input->next != NULL) {
	decode_AssignmentRequestIEs(world, input->next);
    }

    /* process content of current IE */
    switch (opentype->pduNum) {
        case RAB_ReleaseList_PDU:
            ossPrint(world, "\nDecoding opentypes nested in the RAB-ReleaseList ...\n");
	    decode_ReleaseItemIEs(world, (*opentype->decoded.pdu_RAB_ReleaseList)->value);
	    next = (*opentype->decoded.pdu_RAB_ReleaseList)->next;
	    while (next != NULL) {
		decode_ReleaseItemIEs(world, next->value);
		next = next->next;
	    }
	    break;
        case RAB_SetupOrModifyList_PDU:
            ossPrint(world, "\nDecoding opentypes nested in the RAB-SetupOrModifyList ...\n");
	    decode_ContainerPair(world, (*opentype->decoded.pdu_RAB_SetupOrModifyList)->value);
	    next_pair = (*opentype->decoded.pdu_RAB_SetupOrModifyList)->next;
	    while (next_pair != NULL) {
		decode_ContainerPair(world, next_pair->value);
		next_pair = next_pair->next;
	    }
	    break;
	case RAB_ReleaseItem_PDU:
	    /* no opentype nested - nothing to do */
	    break;
	/* more checks for other possible types than can occur in IE
	   ommited for brevity.*/
	default:
	    ossPrint(world, "Error: unknown PDU number\n");
    }
    return;
}

/*
 * Function:            decode_ReleaseItemIEs() decodes protocolIEs PDU
 *
 * Parameters:
 *
 *   "world"        ... pointer to OssGlobal structure.
 *   "input"        ... pointer to the list of protocolIEs to decode
 *
 * Return value:        none.
 */
void decode_ReleaseItemIEs(OssGlobal *world, struct _seqof87 *input)
{
    /* pointer to opentype to decode */
    RAB_ReleaseItemIEs_Value *opentype = &input->value.value;

    /* number of PDU to decode */
    int		pdunum;

    /* decoder input buffer */
    OssBuf	buf;

    buf.length = opentype->encoded.length;
    buf.value = opentype->encoded.value;
    pdunum = RAB_ReleaseItem_PDU;
    ossDecode(world, &pdunum, &buf, (void **)&opentype->decoded);
    opentype->pduNum = pdunum;
    
    /* process content of current IE */
    switch (opentype->pduNum) {
	case RAB_ReleaseItem_PDU:
	    /* no opentype nested - nothing to do */
	    break;
	/* more checks for other possible types than can occur in IE
	   ommited for brevity.*/
	default:
	    ossPrint(world, "Error: unknown PDU number\n");
    }
    return;
}

/*
 * Function:            decode_ContainerPair decodes ContainerPair PDU
 *
 * Parameters:
 *
 *   "world"        ... pointer to OssGlobal structure.
 *   "input"        ... ProtocolIE_ContainerPair PDU to decode
 *
 * Return value:        none.
 */
void decode_ContainerPair(OssGlobal *world, ProtocolIE_ContainerPair input)
{
    /* pointers to opentype to decode */
    RAB_SetupOrModifyItem_IEs_FirstValue *first = &input->value.firstValue;
    RAB_SetupOrModifyItem_IEs_SecondValue *second = &input->value.secondValue;

    /* number of PDU to decode */
    int		pdunum = RAB_SetupOrModifyItemFirst_PDU;

    /* decoder input buffer */
    OssBuf	buf;

    buf.length = first->encoded.length;
    buf.value = first->encoded.value;

    ossDecode(world, &pdunum, &buf, (void **)&first->decoded);
    first->pduNum = pdunum;

    buf.length = second->encoded.length;
    buf.value = second->encoded.value;
    pdunum = RAB_SetupOrModifyItemSecond_PDU;

    ossDecode(world, &pdunum, &buf, (void **)&second->decoded);
    second->pduNum = pdunum;

    return;
}
